'Lib.vbs
'Script library
'Chacal 2010

Function BackupFile (file)
   'Copy a file to Backup subdir in the backup folder
   Dim filesys, path, newFolder
   BackupFile = False
   'Create backup subfolder if needed
   path = ".\Backup\"
   set filesys=CreateObject("Scripting.FileSystemObject")
   If Not filesys.FolderExists(path) Then
      Set newFolder = filesys.CreateFolder(path)
      WriteLog "    BackupFile: Creating backup folder " & path
      Set newFolder = Nothing
   End If
   Set filesys=CreateObject("Scripting.FileSystemObject")
   If filesys.FileExists(file) Then
      WriteLog "    BackupFile: Copying file " & file & " to " & path
      filesys.CopyFile file, path, True
      BackupFile = True
   Else
      WriteLog  "   BackupFile: File not found: " & file
   End If
   Set filesys = Nothing
End Function

Function CopyFileFromGame (subDir,fileName) 
   'copies a file from a Uru game subdirectory to the distro folder 
   Dim filesys, result, path 
   path = gamePath & "\" & subDir & "\" & fileName 
   Set filesys=CreateObject("Scripting.FileSystemObject") 
   If filesys.FileExists(path) Then 
      WriteLog "    CopyFileFromGame: Copying file " & path 
      filesys.CopyFile path, currentDirectory & "\" ,1 
      CopyFileFromGame = True 
   Else 
      result = "    CopyFileFromGame: File not found: " & path
      WriteLog result 
      CopyFileFromGame = False 
   End If 
   Set filesys = Nothing 
   End Function

Function CopyFileToGame (subDir,fileName)
   'copies a file from the distro folder to a Uru game subdirectory
   Dim filesys, path, result
   set filesys=CreateObject("Scripting.FileSystemObject")
   If filesys.FileExists(fileName) Then
      path = gamePath & "\" & subDir & "\"
      WriteLog "    CopyFileToGame: Copying file " & fileName & " to " & path
      filesys.CopyFile fileName, path,1
      CopyFileToGame = True
   Else
      result = "    CopyFileToGame: File not found: " & fileName
      WriteLog result
      CopyFileToGame = False
   End If
   Set filesys = Nothing
End Function

Function DeleteDrawable (spanName,spanIndex)
   'Delete all the icicles (visible meshes) from a drawable
   Dim outputFileName, filesys, currNode, xpath, indexNode, indexNodes, icicleIndex, icicleNode, result, parentNode
   outputFileName = "temp.prc"
   DeleteDrawable = True
   'First Extract the plDrawableSpan to an xml file
   WriteLog "        DeleteDrawable: extracting object " & spanName & " from " & prpFileName
   If Not prpExtractObj (prpFileName , "plDrawableSpans" , spanName, outputFileName) Then
   WriteLog "        DeleteDrawable: can't extract object " & spanName & " from " & prpFileName
      DeleteDrawable = False
   End If
   'Normalize the XML file (escape ampersands) because libPlasma outputs bad XML
   If Not NormalizeXmlFile (outputFileName,True) Then
      DeleteDrawable = False
      Exit Function
   End If
  'Then load into DOM object
   If Not GetXmlDoc (outputFileName) Then
      WriteLog "        DeleteDrawable: can't read XML doc from file " & outputFileName
      DeleteDrawable = False
   End If
   If Not DeleteDrawable Then
      Exit Function
   End If
   'Get icicle indices from xmlDoc. Delete each icicle.
   xpath = "/plDrawableSpans/DIIndices/plDISpanIndex[" & CStr(1+CInt(spanIndex)) &"]/Index"
   WriteLog "        DeleteDrawable: Trying xpath: " & xpath
   Set indexNodes = xmlDoc.selectNodes(xpath)
   'Each object is made of one or many icicles, so each node has one or many icicle indices
   'Now we look at each node
   For each indexNode in indexNodes
      'Get icicle index
      icicleIndex = indexNode.getAttribute("value")
      'Delete that icicle
      WriteLog "        DeleteDrawable: Deleting icicle " & icicleindex
      'First we get the icicle node
      xpath = "/plDrawableSpans/Icicles/plIcicle[" & CStr(1+ CInt(icicleIndex)) & "]/Icicle"
      WriteLog "            Trying xpath: " & xpath
      Set icicleNode = xmlDoc.selectSingleNode(xpath)
      Writelog "            Before: StartIdx=" & icicleNode.getAttribute ("StartIdx") & "    Length=" & icicleNode.getAttribute ("Length")
      'Then we change the Length attribute to 0
      icicleNode.setAttribute "Length", "0"
      Writelog "            After: StartIdx=" & icicleNode.getAttribute ("StartIdx") & "    Length=" & icicleNode.getAttribute ("Length")
      'Replace the node in the xml doc
      xpath = "/plDrawableSpans/Icicles/plIcicle[" & CStr(1+ CInt(icicleIndex)) & "]"
      Set parentNode = xmlDoc.selectSingleNode(xpath)
      parentNode.ReplaceChild icicleNode, parentNode.firstChild
      Next
   'Save the xmlDoc to a temp file
   WriteLog "        DeleteDrawable: Saving xmlDoc to temp file."
   xmlDoc.Save outputFileName
   'De-normalize the XML file (escape ampersands) because libPlasma outputs bad XML
   If Not NormalizeXmlFile (outputFileName,False) Then
      DeleteDrawable = False
      Exit Function
   End If
   'Import the temp file back into the prp
   'First, delete the plDrawableSpans object from the prp
   WriteLog "        DeleteDrawable: Deleting object " & spanName & " from " & prpFileName
   If PrpDelObj (prpFileName, "plDrawableSpans", spanName) Then
      WriteLog "            Object deleted"
   Else
      WriteLog "            Error deleting object"
   End If
   'Then import the new plDrawableSpans object to the prp
   WriteLog "        DeleteDrawable: Adding object " & spanName & " to " & prpFileName
   If PrpAddObj (prpFileName, outputFileName) Then
      WriteLog "            Object added"
   Else
      WriteLog "            Error adding object"
   End If
   Set xmlDoc = Nothing
   'Delete the temp file
   DeleteFile (outputFileName)
   WriteLog "        DeleteDrawable: done."
End Function

Function GetConfig (key)
   'returns value of child of node /config/clientconfig
   Dim currNode
   Set currNode = xmlConfig.selectSingleNode("/config/clientconfig/" & key)
   GetConfig = currNode.Text
   Set currNode = Nothing
End Function

Function DeleteFile (fname)
   'Delete a file from current directory
   Dim filesys
   Set filesys=CreateObject("Scripting.FileSystemObject")
   If filesys.FileExists(fname) Then
      filesys.DeleteFile fname
   End If
   Set filesys = Nothing
End Function

Function DeleteObjectDrawables (drawables)
   'Delete each of an object's drawables
   Dim drawable, spanIndex, spanName
   DeleteObjectDrawables = True
   'Loop for each drawable in the drawables object which is an XML node
   For each drawable in drawables
      'Get index and name of drawable span from xml object
      spanIndex = drawable.getAttribute("key")
      spanName = drawable.firstChild.getAttribute("Name")
      WriteLog "    DeleteObjectDrawables: Deleting index " & spanIndex & " in drawableSpan " & spanName
      ' Delete this drawable
      If Not DeleteDrawable (spanName, spanIndex) Then
      	 DeleteObjectDrawables = False
      End If
   Next
End Function
    
Function GetObjectDrawables (diName, drawables)
   'Get name of object's drawables
   'Return the entire "Drawables" node as an object
   Dim outputFileName, filesys, xpath, drawable
   outputFileName = "temp.prc"
   GetObjectDrawables = True
   'First Extract the plDrawInterface to an xml file
   WriteLog "    GetObjectDrawables: extracting plDrawInterface " & diName & " from " & prpFileName
   If Not prpExtractObj (prpFileName , "plDrawInterface" , diName, outputFileName) Then
   WriteLog "    GetObjectDrawables: can't extract object " & diName & " from " & prpFileName
      GetObjectDrawables = False
   End If
   'Read XML file into xmlDoc (a global XML DOM object)
   If Not GetXmlDoc (outputFileName) Then
      WriteLog "    GetObjectSpans: can't read XML doc from file " & outputFileName
      GetObjectDrawables = False
   End If
   If GetObjectDrawables Then
      'Delete the temp file
      DeleteFile (outputFileName)
      'Get drawables from xmlDoc
      xpath = "/plDrawInterface/Drawables/Drawable"
      Set drawables = xmlDoc.selectNodes(xpath)
      For each drawable in drawables
         WriteLog "    GetObjectDrawables: Found drawable span index " & drawable.GetAttribute("key") 
      Next
      Set xmlDoc = Nothing
   End If
End Function

Function GetObjectInterfaces (diName, siName)
   'Get names of object's interfaces
   Dim outputFileName, filesys, currNode, xpath
   outputFileName = "temp.prc"
   GetObjectInterfaces = True
   'First extract the plSceneObject to an xml file
   WriteLog "    GetObjectInterfaces: extracting object " & plSceneObjectName & " from " & prpFileName
   If Not prpExtractObj (prpFileName , "plSceneObject" , plSceneObjectName, outputFileName) Then
      WriteLog "    GetObjectInterfaces: can't extract object " & plSceneObjectName & " from " & prpFileName
      GetObjectInterfaces = False
   End If
   'Read XML file into xmlDoc (a global XML DOM object)
   If Not GetXmlDoc (outputFileName) Then
      WriteLog "    GetObjectInterfaces: can't read XML doc from file " & outputFileName
      GetObjectInterfaces = False
   End If
   If GetObjectInterfaces Then
      'Delete the temp file
      DeleteFile (outputFileName)
      'Get interfaces from xmlDoc
      xpath = "/plSceneObject/DrawInterface/plKey"
      Set currNode = xmlDoc.selectSingleNode(xpath)
      diName = currNode.getAttribute("Name")
      xpath = "/plSceneObject/SimulationInterface/plKey"
      Set currNode = xmlDoc.selectSingleNode(xpath)
      siName = currNode.getAttribute("Name")
      Set xmlDoc = Nothing
      If diName <> "" Then
         WriteLog "    GetObjectInterfaces: Found DrawInterface " & diName
      End If
      If siName <> "" Then
         WriteLog "    GetObjectInterfaces: Found SimulationInterface " & siName
      End If
   End If   
End Function

Function GetObjectPhysical (siName,physName)
   'Get name of object's plGenericPhysical
   Dim outputFileName, filesys, currNode, xpath
   outputFileName = "temp.prc"
   GetObjectPhysical = True
   'First Extract the plSimulationInterface to an xml file
      WriteLog "    GetObjectPhysical: extracting object " & siName & " from " & prpFileName
   If Not prpExtractObj (prpFileName , "plSimulationInterface" , siName, outputFileName) Then
      WriteLog "    GetObjectPhysical: can't extract object " & siName & " from " & prpFileName
      GetObjectPhysical = False
   End If
   'Read XML file into xmlDoc (a global XML DOM object)
   If Not GetXmlDoc (outputFileName) Then
      WriteLog "    GetObjectPhysical: can't read XML doc from file " & outputFileName
      GetObjectPhysical = False
   End If
   If GetObjectPhysical Then
      'Delete the temp file
      DeleteFile (outputFileName)
      'Get physical from xmlDoc
      xpath = "/plSimulationInterface/Physical/plKey"
      Set currNode = xmlDoc.selectSingleNode(xpath)
      physName = currNode.getAttribute("Name")
      Set xmlDoc = Nothing
      WriteLog "    GetObjectPhysical: Found Physical " & physName 
   End If
End Function

Function GetUserConfig (fname)
   'Get the user local config
   'Return false if a path is still empty
   'Define global paths
   Dim result
   WriteLog "    GetUserConfig: Getting user config from file " & fname
   If Not GetXmlConfig (fname) Then
      WriteLog "    GetUserConfig: The config file is corrupted or doesn't exist."
      Exit Function
   End If
   libPlasmaPath = GetConfig ("libplasmapath")
   If (libPlasmaPath = "") Then
      GetUserConfig = False
      WriteLog "    GetUserConfig: Could not find a path to LibPlasma in the config file."
   Else
      GetUserConfig = True
      gamePath = GetConfig ("gamepath")
      currentDirectory = left(WScript.ScriptFullName,(Len(WScript.ScriptFullName))-(len(WScript.ScriptName)))
   End If
End Function

Function GetXmlConfig (fname)
   'reads an entire XML file into the xmlConfig public object
   Dim result, filesys
   Set xmlConfig=CreateObject("Microsoft.XMLDOM")
   xmlConfig.async="false"
   GetXmlConfig = True
   set filesys=CreateObject("Scripting.FileSystemObject")
   If Not filesys.FileExists(fname) Then
      GetXmlConfig = False
      Exit Function
   Else	
      xmlConfig.load(fname)
      If xmlConfig.parseError.errorCode Then
         GetXmlConfig = False
      End If
      xmlConfig.setProperty "SelectionLanguage", "XPath"
   End If
End Function

Function GetXmlDoc (fname)
   'reads an entire XML file into the xmlDoc public object
   Dim result, filesys
   Set xmlDoc=CreateObject("Microsoft.XMLDOM")
   GetXmlDoc = True
   xmlDoc.async = False
   set filesys=CreateObject("Scripting.FileSystemObject")
   If Not filesys.FileExists(fname) Then
      Writelog "        GetXmlDoc: The xml file " & fname & " doesn't exist!"
      GetXmlDoc = False
      Exit Function
   Else	
      xmlDoc.load(fname)
      If xmlDoc.parseError.errorCode Then
      ' Do something with the error.
         result= "        Parse Error: " & vbcrlf & _ 
                      "               Reason = "  & xmlDoc.parseError.reason & vbcrlf & _ 
                      "               Line = "    & xmlDoc.parseError.line & vbcrlf & _ 
                      "               linePos = " & xmlDoc.parseError.linePos & vbcrlf & _ 
                      "               srcText = " & xmlDoc.parseError.srcText & vbcrlf & _ 
                      "               ErrorCode = " & xmlDoc.parseError.ErrorCode & vbcrlf
         WriteLog "        GetXmlDoc: " & result
         GetXmlDoc = False
      End If
   End If
   xmlDoc.setProperty "SelectionLanguage", "XPath"
   Set filesys = Nothing
End Function

Function NormalizeXmlFile (fname,flag)
   'Normalize an XML file (escape ampersands) because libPlasma outputs bad XML
   'If flag = True, normalize
   'If flag = False, denormalize (put ampersands back as they were)
   Dim result, tempString1, tempString2
   'First read the entire file into memory
   NormalizeXmlFile = True
   If Not ReadFileIntoString (fname,tempString1) Then
      WriteLog "        NormalizeXmlFile: Error reading file " & fname
      NormalizeXmlFile = False
      Exit Function
   End If
   If flag Then
      'Then escape ampersands 
      tempString2=Replace(tempString1,"&amp;","&")
      result=Replace(tempString2,"&","&amp;")
   Else
      result=Replace(tempString1,"&amp;","&")
   End If
   'Write result back to temp file
   WriteFileFromString fname,result
End Function

Function PrpAddObj (prpFileName, objFileName)
   'Adds an object from a .po or .prc file to a .prp file
   Dim WshShell, intResult, prpCmd, Command
   prpCmd = GetConfig ("prpcmd")
   command = libPlasmaPath &"\"& prpCmd & " " & prpFileName &" add " &objFileName
   Set WshShell = WScript.CreateObject("WScript.Shell")
   intResult = WshShell.Run (command, 1, TRUE)
   If intResult Then
       PrpAddObj = False
   End If
      PrpAddObj = True
   Set WshShell = Nothing
End Function

Function PrpDelObj (prpFileName, objType, objName)
   'Delete an object from a .prp file
   Dim WshShell, intResult, prpCmd, result
   prpCmd = GetConfig ("prpcmd")
   command = libPlasmaPath &"\"& prpCmd & " " & prpFileName &" del " & objType & ":" & objName
   Set WshShell = WScript.CreateObject("WScript.Shell")
   intResult = WshShell.Run (command, 1, TRUE)
   'Ignore error if object not found
   If intResult > 1 Then
       PrpDelObj = False
   Else
       PrpDelObj = True
   End If
   Set WshShell = Nothing
End Function

Function PrpExtractObj (prpFileName, objType, objName, outputFileName)
   'Extract an object from a .prp file
   Dim WshShell, intResult, prpCmd, result
   prpCmd = GetConfig ("prpcmd")
   command = libPlasmaPath &"\"& prpCmd & " " & prpFileName &" extract " &_
    objType & ":" & objName & " --prc" & " -o " & outputFileName
   Writelog "        PrpExtractObj: Trying " & Command
   Set WshShell = WScript.CreateObject("WScript.Shell")
   intResult = WshShell.Run (command, 1, TRUE)
   If intResult Then
      PrpExtractObj = False
      Writelog "        PrpExtractObj: Error executing " & Command
   Else
      PrpExtractObj = True
   End If
   Set WshShell = Nothing
End Function

Function ReadFileIntoString (fileName,txt)
   'Reads a text file into a string
   Const ForReading = 1
   Dim WshShell, objFSO, objFile
   ReadFileIntoString = False
   Set objFSO = CreateObject("Scripting.FileSystemObject")
   If Not objFSO.FileExists(fileName) Then
      Writelog "    ReadFileIntoString: File doesn't exist: " & fileName
   Else
      On Error Resume Next
      Set objFile = objFSO.OpenTextFile(fileName, ForReading)
      WriteLog "        ReadFileIntoString: Reading text from file " & fileName
      txt = objFile.ReadAll
      If Err.Number Then
         ReadFileIntoString = False
         Writelog "        ReadFileIntoString: Error reading file: " & err.description
      Else 
         ReadFileIntoString = True
         objFile.Close
      End If
      Set ObjFSO = Nothing
      Set ObjFile = Nothing
      On Error Goto 0
   End If
End Function

Function WriteFileFromString (fileName,txt)
   'Writes a string to a text file
   Dim ObjFSO, ObjFile
   Set ObjFso = CreateObject("Scripting.FileSystemObject")
   Set ObjFile = ObjFso.CreateTextFile(fileName)
   WriteLog "        WriteFileFromString: Writing text to file " & fileName
   On Error Resume Next
   ObjFile.Write(txt)
   If Err.Number Then
      WriteFileFromString = False
   Else 
      WriteFileFromString = True
      ObjFile.Close
   End If
   Set ObjFSO = Nothing
   Set ObjFile = Nothing
   On Error Goto 0
End Function


Sub WriteLog(message)
   'appends message to log file; creates file if needed
   Dim objFSO, objFile
   Set objFSO  = CreateObject("Scripting.FileSystemObject")
   Set objFile = objFSO.OpenTextFile(logFileName, 8, True)
   objFile.WriteLine Now() & "    " & message
   objFile.Close
   Set objFile = Nothing
   Set objFSO  = Nothing
End Sub
